﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

/// <summary>
/// Claim Data Access Object
/// </summary>
public class ClaimDAO : DAO
{
    public List<ClaimDTO> getClaimList()
    {
        return getClaimList(null);
    }
/* fetch list of claims which have not yet been approved */
    public List<ClaimDTO> getClaimList(String whereClause)
    {
        if (null == whereClause)
        {
            whereClause = "";
        }
        List<ClaimDTO> foundClaims = new List<ClaimDTO>();
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM claim " + whereClause;
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        while (myDR.Read())
        {
            foundClaims.Add(makeDTO(myDR));
            /* create new DTO and add to list */
        }
        myConn.Close();
        return foundClaims;
    }
/* fetch a claim by ID or key */
    public ClaimDTO getClaim(int claimKey) 
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM claim WHERE (claimID=";
            mySelectStatement += claimKey.ToString() + ")";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        myConn.Close(); 
        if (!myDR.Read()) return null;
        return makeDTO(myDR);
    }
/* fetch a claim by reference number (the first one to match) */
    public ClaimDTO getClaimByRef(String claimRef)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM claim WHERE (reference='";
        mySelectStatement += claimRef + "')";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        if (!myDR.Read()) return null;
        ClaimDTO myClaim = makeDTO(myDR);
        myConn.Close();
        return myClaim;
    }
/* helper method */
    private ClaimDTO makeDTO(SqlDataReader myDR)
    {
        ClaimDTO myClaim = new ClaimDTO();
        myClaim.claimID = myDR.GetInt32(0);
        myClaim.reference = myDR.GetString(1);
        myClaim.description = myDR.GetString(2);
        myClaim.amount = myDR.GetDecimal(3);
        myClaim.isApproved = (myDR.GetString(4).StartsWith("Y"));
        myClaim.claimDate = myDR.GetDateTime(5);
        myClaim.policyID = myDR.GetInt32(6);
        return myClaim;
    }
/* save a claim object to the database */
    public void saveClaim(ClaimDTO myClaim)
    {
        char isApproved = 'N';
        if (myClaim.isApproved == true)
        {
            isApproved = 'Y';
        }
        String claimDate = myClaim.claimDate.Day + "/" + myClaim.claimDate.Month + "/" + myClaim.claimDate.Year;
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        string insertString = "INSERT INTO claim (reference, description, amount, approved, claimDate, policyID) " +
            "VALUES(@reference, @description, @amount, @approved, @claimDate, @policyID)";
        SqlCommand myComm = new SqlCommand(insertString, myConn);

        myComm.Parameters.AddWithValue("@reference", myClaim.reference);
        myComm.Parameters.AddWithValue("@description", myClaim.description);
        myComm.Parameters.AddWithValue("@amount", myClaim.amount);
        myComm.Parameters.AddWithValue("@approved", isApproved);
        myComm.Parameters.AddWithValue("@claimDate", myClaim.claimDate);
        myComm.Parameters.AddWithValue("@policyID", myClaim.policyID);
        
        int rows = myComm.ExecuteNonQuery();
        myConn.Close();
        if (rows != 1) throw new Exception("Unexpected Result for Insert");
     }
    public void linkToPolicy(ClaimDTO claim, PolicyDTO myPolicy)
    {
        String reference = claim.reference;
        String policyNumber = myPolicy.policyNumber.ToString();
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT policyID FROM policy WHERE (policyNumber=";
        mySelectStatement += policyNumber.ToString() + ")";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        if (!myDR.Read()) return;
        myPolicy.policyID = myDR.GetInt32(0);
        myConn.Close();
        myConn.Open();
        myComm = new SqlCommand("UPDATE claim SET policyID = @policyID WHERE reference = @reference",
            myConn);
        myComm.Parameters.AddWithValue("@policyID", myPolicy.policyID);
        myComm.Parameters.AddWithValue("@reference", claim.reference);
        int rows = myComm.ExecuteNonQuery();
        myConn.Close();
        if (rows != 1) throw new Exception("Unexpected Result for Update");
    }
    
    /* update claim object in the database */
    public void updateClaim(ClaimDTO myClaim)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        SqlCommand myComm = new SqlCommand("UPDATE claim SET reference = @reference, description = @description, amount = @amount, approved = @approved, claimDate = @claimDate, policyID = @policyID WHERE claimID = @claimID",
            myConn);
        myComm.Parameters.AddWithValue("@claimID", myClaim.claimID);
        myComm.Parameters.AddWithValue("@reference", myClaim.reference);
        myComm.Parameters.AddWithValue("@description", myClaim.description);
        myComm.Parameters.AddWithValue("@amount", myClaim.amount);
        myComm.Parameters.AddWithValue("@approved", (myClaim.isApproved ? "Y" : "N"));
        myComm.Parameters.AddWithValue("@claimDate", myClaim.claimDate);
        myComm.Parameters.AddWithValue("@policyID", myClaim.policyID);
        int rows = myComm.ExecuteNonQuery();
        myConn.Close();
        if (rows != 1) throw new Exception("Unexpected Result for Update");
    }

    /* delete claim object in the database */
    public void deleteClaim(ClaimDTO myClaim)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        SqlCommand myComm = new SqlCommand("DELETE FROM claim WHERE claimID = @claimID",
            myConn);
        myComm.Parameters.AddWithValue("@claimID", myClaim.claimID);
        int rows = myComm.ExecuteNonQuery();
        myConn.Close();
        if (rows != 1) throw new Exception("Unexpected Result for Delete");
    }

}
