﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

/// <summary>
/// Policy Data Access Object
/// </summary>
public class PolicyDAO : DAO
{
    public List<PolicyDTO> getPolicyList()
    {
        return getPolicyList(null);
    }

    public List<PolicyDTO> getPolicyList(String whereClause)
    {
        if (null == whereClause)
        {
            whereClause = "";
        }
        List<PolicyDTO> foundPolicies = new List<PolicyDTO>();
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM policy " + whereClause;
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        while (myDR.Read())
        {
            foundPolicies.Add(makeDTO(myDR));
            /* create new DTO and add to list */
        }
        myConn.Close();
        return foundPolicies;
    }
    /* fetch a policy by ID or key */
    public PolicyDTO getPolicyByKey(int policyKey)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM policy WHERE (policyID=";
        mySelectStatement += policyKey.ToString() + ")";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        if (!myDR.Read()) return null;
        myConn.Close();
        return makeDTO(myDR);
    }
    /* fetch a policy by policyNumber */
    public PolicyDTO getPolicyByNumber(int policyNumber)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        String mySelectStatement = "SELECT * FROM policy WHERE (policyNumber=";
        mySelectStatement += policyNumber.ToString() + ")";
        SqlCommand myComm = new SqlCommand(mySelectStatement, myConn);
        SqlDataReader myDR;
        myDR = myComm.ExecuteReader();
        if (!myDR.Read()) return null;
        PolicyDTO myPolicy = makeDTO(myDR);
        myConn.Close();
        return myPolicy; 
    }
    /* helper method */
    private PolicyDTO makeDTO(SqlDataReader myDR)
    {
        PolicyDTO myPolicy = new PolicyDTO();
        myPolicy.policyID = myDR.GetInt32(0);
        myPolicy.policyNumber = myDR.GetInt32(1);
        myPolicy.policyType = myDR.GetString(2);
        myPolicy.startDate = myDR.GetDateTime(3);
        myPolicy.annualPremium = myDR.GetDecimal(4);
        myPolicy.isPaidUp = (myDR.GetString(5).StartsWith("Y"));
        myPolicy.policyHolderID = myDR.GetInt32(6);
        return myPolicy;
    }
    /* save a policy object to the database */
    public void savePolicy(PolicyDTO myPolicy)
    {
        SqlConnection myConn = new SqlConnection(connectionString);
        myConn.Open();
        SqlCommand myComm = new SqlCommand("UPDATE policy SET reference = @reference, description = @description, amount = @amount, approved = @approved, claimDate = @claimDate, policyID = @policyID WHERE claimID = @claimID", 
            myConn);
        myComm.Parameters.AddWithValue("@policyID", myPolicy.policyID);
        myComm.Parameters.AddWithValue("@policyNumber", myPolicy.policyNumber);
        myComm.Parameters.AddWithValue("@policyType", myPolicy.policyType);
        myComm.Parameters.AddWithValue("@startDate", myPolicy.startDate);
        myComm.Parameters.AddWithValue("@annualPremium", myPolicy.annualPremium);
        myComm.Parameters.AddWithValue("@isPaidUp", (myPolicy.isPaidUp ? "Y" : "N"));
        myComm.Parameters.AddWithValue("@policyHolderID", myPolicy.policyID);
        int rows = myComm.ExecuteNonQuery();
        myConn.Close();
        if (rows != 1) throw new Exception("Unexpected Result for Update");
    }

   
}
